@@ -9,7 +9,7 @@ from cosapp.patterns.visitor import Visitor
 from cosapp.core.module import Module
 from cosapp.recorders.recorder import BaseRecorder
 from cosapp.utils.options_dictionary import OptionsDictionary
-from cosapp.utils.naming import NameChecker
+from cosapp.utils.naming import NameChecker, CommonPorts
 from cosapp.utils.helpers import check_arg
 
 logger = logging.getLogger(__name__)
@@ -52,7 +52,7 @@ class Driver(Module):
     _name_check = NameChecker(
         pattern = r"^[A-Za-z][\w\s@-]*[\w]?$",
         message = "Driver name must start with a letter, and contain only alphanumerics + {'_', '@', ' ', '-'}",
-        excluded = ["inwards", "outwards"],
+        excluded = CommonPorts.names(),
     )
 
     def __init__(self,
@@ -122,16 +122,22 @@ class Driver(Module):
 
     @owner.setter
     def owner(self, system: "Optional[cosapp.systems.System]") -> None:
+        self._set_owner(system)
+
+    def _set_owner(self, system: "Optional[cosapp.systems.System]") -> bool:
         from cosapp.systems import System
         if system is not None:
             check_arg(system, 'owner', System)
-
+        
+        changed = system is not self._owner
         self._owner = system
         if self._recorder is not None:
             self._recorder.watched_object = system
         for child in self.children.values():
             child.owner = system
 
+        return changed
+
     def check_owner_attr(self, item: str) -> None:
         if item not in self.owner:
             raise AttributeError(f"{item!r} not found in System {self.owner.name!r}")
