@@ -1,10 +1,62 @@
 import pytest
+import logging, re
 import numpy as np
 
+from cosapp.base import Port, System
 from cosapp.drivers import EulerExplicit
 from cosapp.drivers.time.scenario import Scenario, Interpolator
 from cosapp.core.eval_str import AssignString
 from cosapp.multimode.event import Event
+from cosapp.core.time import UniversalClock
+
+
+@pytest.fixture
+def clock():
+    clock = UniversalClock()
+    yield clock
+    clock.time = 0
+
+
+class XyzPort(Port):
+    def setup(self):
+        self.add_variable('x')
+        self.add_variable('y')
+        self.add_variable('z')
+
+
+class AbcdPort(Port):
+    def setup(self):
+        self.add_variable('a')
+        self.add_variable('b')
+        self.add_variable('c')
+        self.add_variable('d')
+
+
+class XySystem(System):
+    def setup(self):
+        self.add_input(XyzPort, 'x_in')
+        self.add_output(XyzPort, 'x_out')
+
+
+class AbcdSystem(System):
+    def setup(self):
+        self.add_inward('x')
+        self.add_outward('y')
+        self.add_inward('v', np.ones(5))
+        self.add_input(AbcdPort, 'a_in')
+        self.add_output(AbcdPort, 'a_out')
+
+
+@pytest.fixture
+def assembly():
+    """Unconnected assembly of `XySystem` and `AbcdSystem` components"""
+    head = System('head')
+    head.add_child(XySystem('foo1'), pulling='x_in')
+    head.add_child(XySystem('foo2'))
+    head.add_child(AbcdSystem('bar1'), pulling={'v': 'theta'})
+    head.add_child(AbcdSystem('bar2'), pulling={'x': 'alpha', 'a_out': 'a_out'})
+
+    return head
 
 
 @pytest.fixture(scope='function')
@@ -418,3 +470,207 @@ def test_Scenario_clear_values(varying_mu_case):
     scenario.clear_values()
     assert len(scenario.init_values) == 4
     assert len(scenario.case_values) == 0
+
+
+def test_Scenario_aliasing_0(assembly, caplog):
+    head = assembly
+    driver = head.add_driver(EulerExplicit('driver'))
+
+    scenario = Scenario('scenario', driver)
+
+    with caplog.at_level(logging.INFO):
+        scenario.set_init({
+            'foo1.x_in.z': 1.3,
+            'x_in.x': 'x_in.z + 1',
+            'bar2.x': 0.1,  # pulled as `alpha`
+        })
+        scenario.set_values({
+            'foo1.x_in.x': '2 * cos(t)',
+            'foo1.x_in.y': Interpolator(data=[(0, 0), (5, -2), (1000, 500)]),
+            'foo2.x_in.x': 'exp(-t / 4) * foo1.x_in.y',
+            'bar1.v[::2]': '[0.1 * t, 0.2, 0.3]',  # pulled as `theta`
+        })
+    assert len(caplog.records) == 5
+    assert re.match(
+        "Replace 'foo1.x_in.z' by 'x_in.z'",
+        caplog.records[0].message
+    )
+    assert re.match(
+        "Replace 'bar2.x' by 'alpha'",
+        caplog.records[1].message
+    )
+    assert re.match(
+        "Replace 'foo1.x_in.x' by 'x_in.x'",
+        caplog.records[2].message
+    )
+    assert re.match(
+        "Replace 'foo1.x_in.y' by 'x_in.y'",
+        caplog.records[3].message
+    )
+    assert re.match(
+        "Replace 'bar1.v' by 'theta'",
+        caplog.records[4].message
+    )
+    
+    init_conds = map(str, scenario.init_values)
+    time_conds = map(str, scenario.case_values)
+
+    assert set(init_conds) == {
+        'x_in.z = 1.3',
+        'x_in.x = x_in.z + 1',
+        'alpha = 0.1',
+    }
+    assert set(time_conds) == {
+        'x_in.x = 2 * cos(t)',
+        'x_in.y = Interpolator(t)',
+        'foo2.x_in.x = exp(-t / 4) * foo1.x_in.y',
+        'theta[::2] = [0.1 * t, 0.2, 0.3]',
+    }
+
+
+def test_Scenario_aliasing_1(assembly, caplog):
+    """Same as test 0, with additional connections.
+    """
+    head = assembly
+    head.connect(head.foo1.x_out, head.foo2.x_in)
+    head.connect(head.foo2.x_out, head.bar1.a_in, dict(zip('xyz', 'abc')))
+    driver = head.add_driver(EulerExplicit('driver'))
+
+    scenario = Scenario('scenario', driver)
+
+    with caplog.at_level(logging.INFO):
+        scenario.add_init({
+            'foo1.x_in.z': 1.3,
+            'x_in.x': 'x_in.z + 1',
+            'bar2.x': 0.1,  # pulled as `alpha`
+        })
+        scenario.add_values({
+            'foo1.x_in.y': '2 * cos(t)',
+            'bar1.v[::2]': '[0.1 * t, 0.2, 0.3]',  # pulled as `theta`
+        })
+    assert len(caplog.records) == 4
+    assert re.match(
+        "Replace 'foo1.x_in.z' by 'x_in.z'",
+        caplog.records[0].message
+    )
+    assert re.match(
+        "Replace 'bar2.x' by 'alpha'",
+        caplog.records[1].message
+    )
+    assert re.match(
+        "Replace 'foo1.x_in.y' by 'x_in.y'",
+        caplog.records[2].message
+    )
+    assert re.match(
+        "Replace 'bar1.v' by 'theta'",
+        caplog.records[3].message
+    )
+    # Add connected inputs to scenario
+    with pytest.warns(UserWarning, match="Skip connected variable 'bar1.a_in.a'"):
+        scenario.add_init({
+            'bar1.a_in.a': 0.5,  # connected
+            'bar1.a_in.d': 2.5,  # not connected
+        })
+    with pytest.warns(UserWarning, match="Skip connected variable 'bar1.a_in.b'"):
+        scenario.add_values({
+            'bar1.a_in.b': 'sin(t)',
+        })
+    with pytest.warns(UserWarning, match="Skip connected variable 'foo2.x_in.x'"):
+        scenario.add_values({
+            'foo2.x_in.x': Interpolator(data=[(0, 0), (5, -2), (1000, 500)]),
+        })
+    
+    init_conds = map(str, scenario.init_values)
+    time_conds = map(str, scenario.case_values)
+
+    assert set(init_conds) == {
+        'x_in.z = 1.3',
+        'x_in.x = x_in.z + 1',
+        'alpha = 0.1',
+        'bar1.a_in.d = 2.5',
+    }
+    assert set(time_conds) == {
+        'x_in.y = 2 * cos(t)',
+        'theta[::2] = [0.1 * t, 0.2, 0.3]',
+    }
+
+
+def test_Scenario_aliasing_2(assembly, caplog):
+    """Same as test 0, using `ExplicitTimeDriver.set_scenario`
+    instead of `Scenario.set_init` and `set_values`.
+    """
+    head = assembly
+    driver = head.add_driver(EulerExplicit('driver'))
+    with caplog.at_level(logging.INFO):
+        driver.set_scenario(
+            init = {
+                'foo1.x_in.z': 1.3,
+                'x_in.x': 'x_in.z + 1',
+                'bar2.x': 0.1,  # pulled as `alpha`
+            },
+            values = {
+                'foo1.x_in.y': '2 * cos(t)',
+                'foo2.x_in.x': 'exp(-t / 4) * foo1.x_in.y',
+                'bar1.v[::2]': '[0.1 * t, 0.2, 0.3]',  # pulled as `theta`
+            }
+        )
+    assert set(record.message for record in caplog.records) == {
+        "Replace 'foo1.x_in.y' by 'x_in.y' in time scenario.",
+        "Replace 'foo1.x_in.z' by 'x_in.z' in time scenario.",
+        "Replace 'bar1.v' by 'theta' in time scenario.",
+        "Replace 'bar2.x' by 'alpha' in time scenario.",
+    }
+    scenario: Scenario = driver.scenario  # tested object
+    
+    init_conds = map(str, scenario.init_values)
+    time_conds = map(str, scenario.case_values)
+
+    assert set(init_conds) == {
+        'x_in.z = 1.3',
+        'x_in.x = x_in.z + 1',
+        'alpha = 0.1',
+    }
+    assert set(time_conds) == {
+        'x_in.y = 2 * cos(t)',
+        'foo2.x_in.x = exp(-t / 4) * foo1.x_in.y',
+        'theta[::2] = [0.1 * t, 0.2, 0.3]',
+    }
+
+
+def test_Scenario_aliasing_3(assembly, clock):
+    """Check out-of-context warning.
+    """
+    head = assembly
+    # Add time driver to sub-system `head.bar2`:
+    driver = head.bar2.add_driver(EulerExplicit('driver'))
+    scenario = Scenario('scenario', driver)
+
+    pattern = "Variable 'x' is aliased by 'head.alpha', defined outside the context of 'bar2'"
+
+    with pytest.warns(UserWarning, match=pattern):
+        scenario.add_init({
+            'x': 0.1,  # pulled as `alpha`
+        })
+    with pytest.warns(UserWarning, match=pattern):
+        scenario.add_values({
+            'x': 'cos(t)',  # pulled as `alpha`
+        })
+    
+    init_conds = map(str, scenario.init_values)
+    time_conds = map(str, scenario.case_values)
+
+    assert set(init_conds) == {'x = 0.1'}
+    assert set(time_conds) == {'x = cos(t)'}
+    assert scenario.init_values[0].eval_context is head.bar2
+    assert scenario.case_values[0].eval_context is head.bar2
+
+    head.alpha = 0.0
+    head.bar2.x = -0.5
+    scenario.apply_init_values()
+    assert head.alpha == 0
+    assert head.bar2.x == 0.1
+
+    clock.time = 1
+    scenario.update_values()
+    assert head.alpha == 0
+    assert head.bar2.x == pytest.approx(np.cos(1))
